#
#++
#
#  Copyright Digital Equipment Corporation 1996, 1997. All Rights Reserved.
#
#  Restricted Rights: Use, duplication, or disclosure by the U.S. Government
#  is subject to restrictions as set forth in subparagraph (c)(1)(ii) of DFARS
#  252.227-7013, or in FAR 52.227-19, or in FAR 52.227-14 Alt. III, as
#  applicable.
#
#  This software is proprietary to and embodies the confidential technology of
#  Digital Equipment Corporation. Possession, use, or copying of this software
#  and media is authorized only pursuant to a valid written license from
#  Digital or an authorized sublicensor.
#
#--
#
#++
#
#  MODULE:
#
#      D_IP4.DDL
#
#  ABSTRACT:
#
#	DDL program to DECODE IP V4 packets.
#
#  AUTHORS:
#
#      A A Hart
#
#  DATE:
#
#      12 Jan 1997
#
#  MODIFICATION HISTORY:
#
#
INCLUDE ddlinc.ddl

# CONSTANTs
#
CONSTANT IP_header_length_version = 0,
	 IP_TOS = 1,
	 IP_length = 2,
	 IP_ident = 4,
	 IP_fragment_offset = 6,
	 IP_fragment_offset_mask = 0x1fff,
	 IP_fragment_offset_more = 0x20,
	 IP_fragment_offset_dont = 0x40,
	 IP_fragment_size = 13,
	 IP_time_to_live = 8,
	 IP_protocol = 9,
	 IP_checksum = 10,
	 IP_source_address = 12,
	 IP_destination_address = 16,
	 IP_options = 20;

# IP Fragmentation flags
#
FLAG_TABLE ipFragFlags {
    TITLE = "Fragmentation Flags"
    } =
	0x20 = "More",
	0x40 = "Don't"
        ;

# IP TOS Flags Table.
#
FLAG_TABLE ipTosFlags {
    TITLE = "Type of Service"
    } = 
	0x04 = "R",
	0x08 = "T",
	0x10 = "D",
	0x3 = "?"
	;


# IP Precedence name list
#
NAME_LIST ipPrecedence {
    TITLE = "Precedence"
    } =
	"Routine",		# 0
	"Priority",		# 1 
	"Immediate",		# 2 
	"Flash",		# 3 
	"Flash Override",	# 4 
	"CRITIC/ECP",		# 5 
	"Internetwork Control",	# 6 
	"Network Control"	# 7 
	;


# Name Table
# ipProtocols
NAME_TABLE IpProtocols;
NAME_TABLE IpHeaderOptions;
#General Routing Encapsulation Header
#      0                   1                   2                   3
#      0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |C|R|K|S|s|Recur|A| Flags | Ver |         Protocol Type         |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |    Key (HW) Payload Length    |       Key (LW) Call ID        |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |                  Sequence Number (Optional)                   |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |               Acknowledgment Number (Optional)                |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
CONSTANT
	GRE_flags = 0, 
	GRE_v_sequence = 12,
	GRE_v_acknowledge = 7,
	GRE_protocol_type = GRE_flags+2,
	GRE_length = GRE_protocol_type+2,
	GRE_call_ID = GRE_length+2,
	GRE_seq_no = GRE_call_ID+2,
	GRE_ack_no_with_seq = GRE_seq_no+4,
	GRE_ack_no = GRE_call_ID+2,
;

DECLARE PST ppp_protocols;

DECODE gre_ppp(0x2001880b) {TITLE = "PPP", PLEVEL = HL_PPP, CONSUME = 8, NEXT = ppp_protocols} =
	decimal(word(GRE_length),0,"Payload length"),
	decimal(word(GRE_call_ID),0,"Call ID"){BRIEF},
;
DECODE gre_ppp_seq(0x3001880b) {TITLE = "PPP", PLEVEL = HL_PPP, CONSUME = 12, NEXT = ppp_protocols} =
	decimal(word(GRE_length),0,"Payload length"),
	decimal(word(GRE_call_ID),0,"Call ID"){BRIEF},
	decimal(long(GRE_seq_no),0,"Sequence number"),
;
DECODE gre_ppp_ack(0x2081880b) {TITLE = "PPP", PLEVEL = HL_PPP, CONSUME = 12, NEXT = ppp_protocols} =
	decimal(word(GRE_length),0,"Payload length"),
	decimal(word(GRE_call_ID),0,"Call ID"){BRIEF},
	decimal(long(GRE_ack_no),0,"Acknowledge number"),
;
DECODE gre_ppp_seq_ack(0x3081880b) {TITLE = "PPP", PLEVEL = HL_PPP, CONSUME = 16, NEXT = ppp_protocols} =
	decimal(word(GRE_length),0,"Payload length"),
	decimal(word(GRE_call_ID),0,"Call ID"){BRIEF},
	decimal(long(GRE_seq_no),0,"Sequence number"),
	decimal(long(GRE_ack_no_with_seq),0,"Acknowledge number"),
;
PST gre_types ( long(GRE_flags) ) {TITLE = "GRE protocol"} =
	gre_ppp,
	gre_ppp_seq,
	gre_ppp_ack,
	gre_ppp_seq_ack;


# PST: pst_ip
# Entry point for IP packets.  This PST provides the TYPE for IP so
# all IP packets will be routed through this PST.  The PST switches
# on the IP version field and calls either IPV6 or IPV4 decode.
#
DECLARE PST ipv4_types, ipv6_types;
PST pst_ip( bitfield_byte( 4, 4, IP_header_length_version) ) {
	TITLE = "IP Packet",
	TYPE = "IP"
	} =
	    04 -> ipv4_types { TITLE = "IP", PLEVEL = HL_IP },
	    06 -> ipv6_types { TITLE = "IPV6", PLEVEL = HL_IP}
	    ;


# PST: PST_ip_protocols
# SWITCH on the IP Protocol field.  Punt the ones that we don't want to
# DDL code to DTF.
#
#	$32 = IP header length (in bytes)
#
DECLARE PST icmp_types, ipv4_types, ipv6_types;
DECLARE DECODE d_udp, d_tcp, d_ospf;

PST pst_ip_protocols( byte(IP_protocol) ) {
    TITLE = "IP Protocol"
    } =
	01 -> icmp_types { CONSUME = $32, TITLE = "ICMP", PLEVEL = 0 },
	04 -> ipv4_types { CONSUME = $32, TITLE = "IP-in-IP", PLEVEL = HL_IP },
	41 -> ipv6_types { CONSUME = $32, TITLE = "IPV6", PLEVEL = HL_IP },
	47 -> gre_types  { CONSUME = $32, PLEVEL = HL_IP, BRIEF_TITLE = "GRE" }
	;


# PST: PST_ipv4 (ipv4_types)
# Entry point PST, SWITCHes on the fragment bit.
#
DECLARE DECODE d_ip_common, d_ip_common_fragment;
PST ipv4_types( bitfield_word(IP_fragment_size,0,IP_fragment_offset) ) {
    DEFAULT = d_ip_common_fragment
     } = 
	d_ip_common
	;
#Copy Class Number Value Name				Reference
#   0     0      0     0 EOOL   - End of Options List    [RFC791,JBP] - fixed length of 1 byte
#   0     0      1     1 NOP    - No Operation           [RFC791,JBP] - fixed length of 1 byte
#   1     0      2   130 SEC    - Security                  [RFC1108]
#   1     0      3   131 LSR    - Loose Source Route     [RFC791,JBP]
#   0     2      4    68 TS     - Time Stamp             [RFC791,JBP]
#   1     0      5   133 E-SEC  - Extended Security         [RFC1108]
#   1     0      6   134 CIPSO  - Commercial Security           [???]
#   0     0      7     7 RR     - Record Route           [RFC791,JBP]
#   1     0      8   136 SID    - Stream ID              [RFC791,JBP]
#   1     0      9   137 SSR    - Strict Source Route    [RFC791,JBP]
#   0     0     10    10 ZSU    - Experimental Measurement      [ZSu]
#   0     0     11    11 MTUP   - MTU Probe                 [RFC1191]
#   0     0     12    12 MTUR   - MTU Reply                 [RFC1191]
#   1     2     13   205 FINN   - Experimental Flow Control    [Finn]
#   1     0     14   142 VISA   - Expermental Access Control [Estrin]
#   0     0     15    15 ENCODE - ???                      [VerSteeg]
#   1     0     16   144 IMITD  - IMI Traffic Descriptor        [Lee]
#   1     0     17   145 EIP    - ???			   [RFC1358]
#   0     2     18    82 TR     - Traceroute		   [RFC1393]
#   1     0     19   147 ADDEXT - Address Extension    [Ullmann IPv7]	
#   1     0     20   148 RTRALT - Router Alert              [RFC2113]
#   1     0     21   149 SDB    - Selective Directed Broadcast[Graff]
#   1     0     22   150 NSAPA  - NSAP Addresses          [Carpenter]

DECLARE PST pst_ip_options_left;
DECLARE PST pst_ip_options_decode;

DECODE d_ip_option(0) {
	NEXT = pst_ip_options_left, PLEVEL=HL_IP, TITLE = "Undecoded option"
	} =

		$1 = $32 - $33,			# Offset of next option
		$2 = $1 + 1,			# Offset of length field
		$33 = $33 - byte($2),		# Update unprocessed bytes
		$34 = $32 - $33,		# update option pointer
		name( "Header Option", IpHeaderOptions, byte($1) ),
		dump_hex_bytes( $1 + 2, byte($2) - 2, " option data" )
		;

DECODE d_ip_opt_eool(0) {TITLE = "End of Options List",ATTACH = pst_ip_options_decode, NEXT = pst_ip_options_left 
	} = 
	$33 = 0
	;
DECODE d_ip_opt_nop(1) {TITLE = "No Operation",ATTACH = pst_ip_options_decode, NEXT = pst_ip_options_left
	} = 
	$33 = $33 - 1,
	$34 = $34 + 1
	;
DECLARE PST pst_ip_route_option_left;

DECODE d_ip_route_remaining(0) {NEXT = pst_ip_route_option_left
	} =
	ip_address(long($35),0," address"),
	$35 = $35 + 4,
	$36 = $36 - 4
	;

PST pst_ip_route_option_left ($36) {
	DEFAULT = d_ip_route_remaining
	} =
		0x0 -> pst_ip_options_left;

DECODE d_ip_opt_ssr(137) {TITLE = "Strict Source Route",ATTACH = pst_ip_options_decode, NEXT = pst_ip_route_option_left, PLEVEL=HL_IP
	} =
	$1 = $34 + 1,
	$2 = byte($1),	#$2 becomes the option length
	decimal($2,0," SSR - option length"),
	decimal(byte($34+2),0," SSR - Pointer to next"),
	ip_address(long($34+3),0," address"),
	$33 = $33 - $2,
	$35 = $34 + 7,
	$36 = $2 - 7,
	$34 = $34 + $2
	;
DECODE d_ip_opt_rr(7) {TITLE = "Route Record",ATTACH = pst_ip_options_decode, NEXT = pst_ip_route_option_left, PLEVEL=HL_IP
	} =
	$1 = $34 + 1,
	$2 = byte($1),	#$2 becomes the option length
	decimal($2,0," RR - option length"),
	decimal(byte($34+2),0," RR - Pointer to next"),
	ip_address(long($34+3),0," address"),
	$33 = $33 - $2,
	$35 = $34 + 7,
	$36 = $2 - 7,
	$34 = $34 + $2
	;

DECODE d_ip_opt_lsr(131) {TITLE = "Loose Source Route",ATTACH = pst_ip_options_decode, NEXT = pst_ip_route_option_left, PLEVEL=HL_IP
	} =
	$1 = $34 + 1,
	$2 = byte($1),	#$2 becomes the option length
	decimal($2,0," LSR - option length"),
	decimal(byte($34+2),0," LSR - Pointer to next"),
	ip_address(long($34+3),0," address"),
	$33 = $33 - $2,
	$35 = $34 + 7,
	$36 = $2 - 7,
	$34 = $34 + $2
	;
DECLARE PST pst_ip_ts_time, pst_ip_ts_time_address;

DECODE d_ip_ts_time_next(0) {NEXT = pst_ip_ts_time} 
	= 
	decimal(long($35+4),0,"Time"),
	$35 = $35 + 4,
	$36 = $36 - 4
	;
PST pst_ip_ts_time ($36) {
	DEFAULT = d_ip_ts_time_next
	} =
	0 -> pst_ip_options_left;

DECODE d_ip_ts_timeadd_next(0) {NEXT = pst_ip_ts_time_address} 
	= 
	ip_address(long($35),0," address"),
	decimal(long($35+4),0,"Time"),
	$35 = $35 + 8,
	$36 = $36 - 8
	;

PST pst_ip_ts_time_address ($36) {
	DEFAULT = d_ip_ts_timeadd_next
	} =
	0 -> pst_ip_options_left;

PST pst_ip_ts_option_type ($37) 
	=
	0 -> pst_ip_ts_time,
	1 -> pst_ip_ts_time_address,
	3 -> pst_ip_ts_time_address
	;
DECODE d_ip_opt_ts(68) {TITLE = "Time Stamp",ATTACH = pst_ip_options_decode, NEXT = pst_ip_ts_option_type, PLEVEL=HL_IP
	} =
	$1 = $34 + 1,
	$2 = byte($1),	#$2 becomes the option length
	decimal($2,0," TS - option length"),
	decimal(byte($34+2),0," TS - Pointer to next"),
	$37 = byte($34+3) & 15, #timestamp element format
	decimal(byte($34+3)/16,0," TS - Overflows"),
	$33 = $33 - $2, #option bytes left
	$35 = $34 + 4,	#next timestamp element
	$36 = $2 - 4,	#number of timestamp bytes left
	$34 = $34 + $2	#next option
	;
	
NAME_LIST routerAlerts {
    TITLE = "Alert info",
    OUT_OF_RANGE = "Not defined in RFC2113"
    } =
	"Router must parse";		# 0

DECODE d_ip_opt_ra(148) {TITLE = "Router Alert",ATTACH = pst_ip_options_decode, NEXT = pst_ip_options_left, PLEVEL=HL_IP
	} =
	$1 = $34 + 1,
	$2 = byte($1),	#$2 becomes the option length
	list(word ($34+2), 0,routerAlerts),
	$33 = $33 - $2, #option bytes left
	$34 = $34 + $2	#next option
	;
	
PST pst_ip_options_decode (byte($34)) {
	DEFAULT = d_ip_option,
	TITLE = "IP option"
	} =
  ;

# DECODE: d_ip_option
# Decode an IP option, currently no individual decoding is done, each option
# is printed as a name followed by a hex dump of the option bytes.
# Input:
#	$32 = header length
#	$33 = number of header option bytes unprocessed
#	$34 = offset of next option type
# Output:
#	$33 = number of option bytes remaining
#	$34 = offset of next option type
#
#DECODE d_ip_option(0) {
#	NEXT = pst_ip_options_left
#	} =
#
#		$1 = $32 - $33,			# Offset of next option
#		$2 = $1 + 1,			# Offset of length field
#		$33 = $33 - byte($2),		# Update unprocessed bytes
#		$34 = $32 - $33			# update option pointer
#		name( "Header Option", IpHeaderOptions, byte($1) ),
#		dump_hex_bytes( $1 + 2, $2 - 2, "             " )
#		;

DECODE d_ip_options_decode(0) {
	NEXT = pst_ip_options_decode
	} =
	;
# PST: pst_ip_options_left
# Switch depending upon whether there are any options left to process.
#
PST pst_ip_options_left ($33) {
	DEFAULT = d_ip_options_decode
	} =
		0x0 -> pst_ip_protocols;

#
# DECODE d_ip_common
# DECODE IP header for the first fragment or a non-fragmented packet.
#
# Output:
#	$32 = header length
#	$33 = options length
#	$34 = offset of next option type
#
DECODE d_ip_common(0) {
    NEXT = pst_ip_options_left,
    TITLE = "IP",
    BRIEF_TITLE = "", #IP has already been output at the v4/v6 switch
    PLEVEL = HL_IP
    } = 
	
	# Calculate the IP header length, the length is stored in the low nibble
	# of the first byte as a longword count.
	#
	$32 = bitfield_byte(4,0,IP_header_length_version) * 4,
	$33 = $32 - IP_options,
	$34 = $32 - $33,
	decimal( $32, 0, "Header Length" ),

	flags( byte(IP_TOS), 0, ipTosFlags ),
	list( bitfield_byte( 3, 5, IP_TOS), 0, ipPrecedence ),
	length( uword(IP_length), 0, "Total Length" ) {BRIEF},
	hex( word( IP_ident ), 4, "Packet Identifier" ),

	flags( byte(IP_fragment_offset), 0, ipFragFlags ) {CONDITION = byte(IP_fragment_offset)},
	hex( bitfield_word(IP_fragment_size,0,IP_fragment_offset), 4, "Fragment Offset" ),
	decimal( byte(IP_time_to_live), 0, "Time to live" ),

	# The IP_CHECKSUM() routine wants the packet length in the high word and 
	# offset of the checksum field in the low word, the length is taken from the
	# IP header.
	ip_checksum( IP_checksum, ((byte(IP_header_length_version) & 0xf) << 18) + IP_checksum, "Verified Checksum" ),
	ip_address( long(IP_source_address), 0, "Source Address" ) {BRIEF},
	ip_address( long(IP_destination_address), 0, "Destination Address" ) {BRIEF},
	name( "Protocol", IpProtocols, byte(IP_protocol) ),
	;


#
# DECODE d_ip_common_fragment
# DECODE IP header for the fragment and then exit (we don't know what's
# in the rest of the packet).
#
DECODE d_ip_common_fragment(0) {
    TITLE = "IP fragment",
    PLEVEL = HL_IP
    } = REF d_ip_common
	;
