#
#++
#
#  Copyright Digital Equipment Corporation 1996, 1997. All Rights Reserved.
#
#  Restricted Rights: Use, duplication, or disclosure by the U.S. Government
#  is subject to restrictions as set forth in subparagraph (c)(1)(ii) of DFARS
#  252.227-7013, or in FAR 52.227-19, or in FAR 52.227-14 Alt. III, as
#  applicable.
#
#  This software is proprietary to and embodies the confidential technology of
#  Digital Equipment Corporation. Possession, use, or copying of this software
#  and media is authorized only pursuant to a valid written license from
#  Digital or an authorized sublicensor.
#
#--
#
#++
#
#  MODULE:
#
#      D_rpf.DDL
#
#  ABSTRACT:
#
#	DDL program to DECODE Route Propogation and Filtering events
#
#  AUTHORS:
#
#      G Mills
#
#  DATE:
#
#      24 Oct 1997
#
#  MODIFICATION HISTORY:
#
#
INCLUDE ddlinc.ddl

# CONSTANTs
#
CONSTANT RPF_filter_address = 0,
	 RPF_filter_mask = RPF_filter_address+4,
	 RPF_filter_status = RPF_filter_mask+4,
	 RPF_filter_route_attr = RPF_filter_status+4,
	 RPF_filter_preference = RPF_filter_route_attr+4,
	 RPF_filter_cp_type = RPF_filter_preference+4,
	 RPF_filter_length = RPF_filter_cp_type+4,

	 RPF_prop_address = 0,
	 RPF_prop_mask = RPF_prop_address+4,
	 RPF_prop_status = RPF_prop_mask+4,
	 RPF_prop_route_attr = RPF_prop_status+4,
	 RPF_prop_source_type = RPF_prop_route_attr+4,
	 RPF_prop_dest_type = RPF_prop_source_type+4,
	 RPF_prop_announce_metric = RPF_prop_dest_type+4,
	 RPF_prop_ann_met_class = RPF_prop_announce_metric+4,
	 RPF_prop_length = RPF_prop_ann_met_class+4;

NAME_LIST rpf_query_status {
    TITLE = "Query Status"
    } =
	"BLOCK",		# 0
	"PROPOGATE",		# 1 
	"Invalid value"
	;
NAME_LIST rpf_filter_status {
    TITLE = "Filter Status"
    } =
	"FILTER",		# 0
	"RECEIVED",		# 1 
	"Invalid value"
	;

NAME_LIST rpf_cpro_types {
    TITLE = "Control Protocol"
    } =
	"Invalid - ZERO",
	"ISIS - Level 1", #1
	"ISIS - Level 2", #2
	"Routing Vector L1", #3
	"Routing Vector L2", #4
	"RIP", #5
	"EGP", #6
	"OSPF General", #7
	"OSPF Area", #8
	"Netware RIP", #9
	"Netware SAP", #10
	"XNS RIP", #11
	"APPLETALK rtmp", #12
	"APPLETALK zip", #13
	"Static", #14
	"Local", #15
	"Illegal";

NAME_LIST rpf_sra_types {
    TITLE = "Source Route Attribute"
    } =
	"Invalid - ZERO",
	"None", #1
	"ISIS - Level 2 INTERNAL", #2
	"ISIS - Level 2 EXTERNAL", #3
	"OSPF INTERNAL", #4
	"OSPF SUMMARY", #5
	"OSPF External 1",#6
	"OSPF External 2",#7
	"Static INTERNAL", #8
	"Static EXTERNAL", #9
	"Illegal";

NAME_LIST rpf_metrics {
    TITLE = "Metric"
    } =
	"None", #0
	"None", #1
	"ISIS Level 2 INTERNAL", #2
	"ISIS Level 2 EXTERNAL", #3
	"OSPF Internal", #4
	"OSPF Summary", #5
	"OSPF External 1",#6
	"OSPF External 2",#7
	"OSPF Intra area",#8
	"OSPF Inter area",#9
	"Illegal - TEN",#10
	"Illegal - 11",#11
	"Illegal - 12",#12
	"Illegal - 13",#13
	"OSPF static internal", #14
	"OSPF static external", #15
	"illegal";


DECODE d_rpf_prop (0) {CONSUME = RPF_prop_length } =
		ip_address( le_long(RPF_prop_address),0,"Address" ) {BRIEF},
		ip_address( le_long(RPF_prop_mask),0,"Mask" ),
		list(le_long(RPF_prop_route_attr), 0 ,rpf_sra_types),
		list(le_long(RPF_prop_status), 0 ,rpf_query_status),
		list(le_long(RPF_prop_source_type), 0 ,rpf_cpro_types),
		list(le_long(RPF_prop_dest_type), 0 ,rpf_cpro_types),
		list(le_long(RPF_prop_announce_metric), 0 ,rpf_metrics),
		hex(le_long(RPF_prop_ann_met_class), 0 ,"Metric Class"),
		;

PST pst_rpf_prop ( 0 ) {
	TITLE = "RPF Propogation",
	TYPE = "RPF_PROP",
	DEFAULT = d_rpf_prop
	} = ;

DECODE d_rpf_filter (0) {CONSUME = RPF_filter_length } =
		ip_address( le_long(RPF_filter_address),0,"Address" ) {BRIEF},
		ip_address( le_long(RPF_filter_mask),0,"Mask" ),
		list(le_long(RPF_filter_route_attr), 0 ,rpf_sra_types),
		list(le_long(RPF_filter_status), 0 ,rpf_filter_status),
		list(le_long(RPF_filter_cp_type), 0 ,rpf_cpro_types),
		hex(le_long(RPF_filter_preference), 0 ,"Preference"),
		;

PST pst_rpf_filter ( 0 ) {
	TITLE = "RPF filter",
	TYPE = "RPF_FILTER",
	DEFAULT = d_rpf_filter
	} = ;
