#
#++
#
#  Copyright Digital Equipment Corporation 1996, 1997. All Rights Reserved.
#
#  Restricted Rights: Use, duplication, or disclosure by the U.S. Government
#  is subject to restrictions as set forth in subparagraph (c)(1)(ii) of DFARS
#  252.227-7013, or in FAR 52.227-19, or in FAR 52.227-14 Alt. III, as
#  applicable.
#
#  This software is proprietary to and embodies the confidential technology of
#  Digital Equipment Corporation. Possession, use, or copying of this software
#  and media is authorized only pursuant to a valid written license from
#  Digital or an authorized sublicensor.
#
#--
#
#++
#
#  MODULE:
#
#      D_UDP.DDL
#
#  ABSTRACT:
#
#	DDL program to DECODE UDP packets.
#
#  AUTHORS:
#
#      A A Hart
#
#  DATE:
#
#      13 Feb 1997
#
#  MODIFICATION HISTORY:
#
#	11-Jul-1997	A A Hart
#	Move the TYPES="UDP" to pst_tcp from pst_udps to allow the UDP
#	header to be decoded.
#
INCLUDE ddlinc.ddl

# CONSTANTs
#
CONSTANT UDP_source_port = 0,
	 UDP_destination_port = 2,
  	 UDP_length = 4,
	 UDP_checksum = 6;


# Name Table
# UDP Ports
NAME_TABLE UdpPorts {
    TITLE = "UDP Port"
    };


# PST: pst_udps
# Main entry point for UDP packets.  This PST switches on the source port,
# if it doesn't find a match it calls the pst_udpd pst to switch on the
# destination UDP port.
#
# CO = UDP Header
#
DECLARE PST pst_udpd;
DECODE d_udps(0) { NEXT = pst_udpd, PLEVEL = HL_UDP } = ;
PST pst_udps( word(UDP_source_port) ) {
	TITLE = "UDP port",
	DEFAULT = d_udps
	} =
		;


# PST: pst_udpd
# Secondary entry point for UDP packets.  This PST switches on the destination
# port, if it doesn't find a match it calls the default_udp_types builtin.
#
# CO = UDP Header
#
DECLARE PST default_udp_types;
DECODE d_udpd(0) { NEXT = default_udp_types } = ;
PST pst_udpd( word(UDP_destination_port) ) {
	TITLE = "UDP port",
	DEFAULT = d_udpd
	} =
		;


# DECODE: d_udp4
# Output the IP V4 UDP header, the checksum is not output.
#
# CO = UDP Header
#
DECODE d_udp4(0) {
	NEXT = pst_udps
	} = 
		decimal( word(UDP_length), 0, "UDP length" ),
		name( "Source port", UdpPorts, word(UDP_source_port) ),
		name( "Destination port", UdpPorts, word(UDP_destination_port) )
		;


# DECODE: d_udp
# Verify and output the UDP checksum and then consume the IP header and
# invoke the main UDP PST.
#
# CO = IP V4 header.
# $32 = IP header length (in bytes)
#
DECLARE PST pst_udp, pst_ip_protocols;
DECODE d_udp(17) {
	TITLE = "UDP",
	ATTACH = pst_ip_protocols,
	PLEVEL = HL_UDP,
	NEXT = pst_udp,
	CONSUME = $32
	} =
	header_checksum( UDP_checksum, 0, "Verified UDP checksum" )
	;


# PST: pst_udp
# Main PST for UDP, this simply invokes the d_udp4 decode.
#
PST pst_udp(0) { DEFAULT = d_udp4, TYPE = "UDP" } = ;



